@@ -27,6 +27,7 @@
  * Interface to new debugger.
  */
 
+#include <string.h>
 #include <sys/reboot.h>
 #include <vm/pmap.h>
 
@@ -43,6 +44,7 @@
 
 #include "vm_param.h"
 #include <vm/vm_map.h>
+#include <vm/vm_fault.h>
 #include <kern/cpu_number.h>
 #include <kern/printf.h>
 #include <kern/thread.h>
@@ -59,8 +61,10 @@
 
 #if MACH_KDB
 /* Whether the kernel uses any debugging register.  */
-static int kernel_dr;
+static boolean_t kernel_dr;
 #endif
+/* Whether the current debug registers are zero.  */
+static boolean_t zero_dr;
 
 void db_load_context(pcb_t pcb)
 {
@@ -72,13 +76,20 @@ void db_load_context(pcb_t pcb)
 		return;
 	}
 #endif
+	/* Else set user debug registers, if any */
+	unsigned int *dr = pcb->ims.ids.dr;
+	boolean_t will_zero_dr = !dr[0] && !dr[1] && !dr[2] && !dr[3] && !dr[7];
+
+	if (!(zero_dr && will_zero_dr))
+	{
+		set_dr0(dr[0]);
+		set_dr1(dr[1]);
+		set_dr2(dr[2]);
+		set_dr3(dr[3]);
+		set_dr7(dr[7]);
+		zero_dr = will_zero_dr;
+	}
 
-	/* Else set user debug registers */
-	set_dr0(pcb->ims.ids.dr[0]);
-	set_dr1(pcb->ims.ids.dr[1]);
-	set_dr2(pcb->ims.ids.dr[2]);
-	set_dr3(pcb->ims.ids.dr[3]);
-	set_dr7(pcb->ims.ids.dr[7]);
 #if MACH_KDB
 	splx(s);
 #endif
@@ -135,7 +146,7 @@ void db_dr (
 		splx(s);
 		return;
 	    }
-	    kernel_dr = 1;
+	    kernel_dr = TRUE;
 	    /* Clear user debugging registers */
 	    set_dr7(0);
 	    set_dr0(0);
@@ -161,7 +172,9 @@ void db_dr (
 	if (kernel_dr) {
 	    if (!ids.dr[0] && !ids.dr[1] && !ids.dr[2] && !ids.dr[3]) {
 		/* Not used any more, switch back to user debugging registers */
-		kernel_dr = 0;
+		set_dr7 (0);
+		kernel_dr = FALSE;
+		zero_dr = TRUE;
 		db_load_context(current_thread()->pcb);
 	    }
 	}
@@ -170,12 +183,12 @@ void db_dr (
 
 boolean_t
 db_set_hw_watchpoint(
-	db_watchpoint_t	watch,
-	unsigned	num)
+	const db_watchpoint_t	watch,
+	unsigned		num)
 {
 	vm_size_t	size = watch->hiaddr - watch->loaddr;
 	db_addr_t	addr = watch->loaddr;
-	unsigned int kern_addr;
+	vm_offset_t 	kern_addr;
 
 	if (num >= 4)
 	    return FALSE;
@@ -214,8 +227,9 @@ db_clear_hw_watchpoint(
  * Print trap reason.
  */
 void
-kdbprinttrap(type, code)
-	int	type, code;
+kdbprinttrap(
+	int	type, 
+	int	code)
 {
 	printf("kernel: %s (%d), code=%x\n",
 		trap_name(type), type, code);
@@ -232,7 +246,7 @@ boolean_t
 kdb_trap(
 	int	type,
 	int	code,
-	register struct i386_saved_state *regs)
+	struct i386_saved_state *regs)
 {
 	spl_t	s;
 
@@ -408,12 +422,12 @@ boolean_t db_no_vm_fault = TRUE;
 
 int
 db_user_to_kernel_address(
-	task_t		task,
+	const task_t	task,
 	vm_offset_t	addr,
-	unsigned int	*kaddr,
+	vm_offset_t	*kaddr,
 	int		flag)
 {
-	register pt_entry_t *ptp;
+	pt_entry_t *ptp;
 	boolean_t	faulted = FALSE;
 
 	retry:
@@ -437,7 +451,7 @@ db_user_to_kernel_address(
 	    }
 	    return(-1);
 	}
-	*kaddr = (unsigned)ptetokv(*ptp) + (addr & (INTEL_PGBYTES-1));
+	*kaddr = ptetokv(*ptp) + (addr & (INTEL_PGBYTES-1));
 	return(0);
 }
 
@@ -448,13 +462,13 @@ db_user_to_kernel_address(
 void
 db_read_bytes(
 	vm_offset_t	addr,
-	register int	size,
-	register char	*data,
+	int		size,
+	char		*data,
 	task_t		task)
 {
-	register char	*src;
-	register int	n;
-	unsigned	kern_addr;
+	char		*src;
+	int		n;
+	vm_offset_t	kern_addr;
 
 	src = (char *)addr;
 	if ((addr >= VM_MIN_KERNEL_ADDRESS && addr < VM_MAX_KERNEL_ADDRESS) || task == TASK_NULL) {
@@ -491,19 +505,18 @@ db_read_bytes(
 void
 db_write_bytes(
 	vm_offset_t	addr,
-	register int	size,
-	register char	*data,
+	int		size,
+	char		*data,
 	task_t		task)
 {
-	register char	*dst;
+	char		*dst;
 
-	register pt_entry_t *ptep0 = 0;
+	pt_entry_t *ptep0 = 0;
 	pt_entry_t	oldmap0 = 0;
 	vm_offset_t	addr1;
-	register pt_entry_t *ptep1 = 0;
+	pt_entry_t *ptep1 = 0;
 	pt_entry_t	oldmap1 = 0;
 	extern char	etext;
-	void		db_write_bytes_user_space();
 
 	if ((addr < VM_MIN_KERNEL_ADDRESS) ^
 	    ((addr + size) <= VM_MIN_KERNEL_ADDRESS)) {
@@ -560,13 +573,13 @@ db_write_bytes(
 void
 db_write_bytes_user_space(
 	vm_offset_t	addr,
-	register int	size,
-	register char	*data,
+	int		size,
+	char		*data,
 	task_t		task)
 {
-	register char	*dst;
-	register int	n;
-	unsigned	kern_addr;
+	char		*dst;
+	int		n;
+	vm_offset_t	kern_addr;
 
 	while (size > 0) {
 	    if (db_user_to_kernel_address(task, addr, &kern_addr, 1) < 0)
@@ -585,7 +598,7 @@ db_write_bytes_user_space(
 boolean_t
 db_check_access(
 	vm_offset_t	addr,
-	register int	size,
+	int		size,
 	task_t		task)
 {
 	int	n;
@@ -616,7 +629,7 @@ boolean_t
 db_phys_eq(
 	task_t		task1,
 	vm_offset_t	addr1,
-	task_t		task2,
+	const task_t	task2,
 	vm_offset_t	addr2)
 {
 	vm_offset_t	kern_addr1, kern_addr2;
@@ -639,16 +652,19 @@ db_phys_eq(
 #define DB_USER_STACK_ADDR		(VM_MIN_KERNEL_ADDRESS)
 #define DB_NAME_SEARCH_LIMIT		(DB_USER_STACK_ADDR-(INTEL_PGBYTES*3))
 
+#define GNU
+
+#ifndef GNU
 static boolean_t
 db_search_null(
-	task_t		task,
+	const task_t	task,
 	vm_offset_t	*svaddr,
 	vm_offset_t	evaddr,
 	vm_offset_t	*skaddr,
 	int		flag)
 {
-	register unsigned vaddr;
-	register unsigned *kaddr;
+	unsigned vaddr;
+	unsigned *kaddr;
 
 	kaddr = (unsigned *)*skaddr;
 	for (vaddr = *svaddr; vaddr > evaddr; ) {
@@ -669,13 +685,12 @@ db_search_null(
 	}
 	return FALSE;
 }
-
-#define GNU
+#endif /* GNU */
 
 #ifdef GNU
 static boolean_t
 looks_like_command(
-	task_t		task,
+	const task_t	task,
 	char*		kaddr)
 {
 	char *c;
@@ -715,19 +730,19 @@ looks_like_command(
 
 	return TRUE;
 }
-#endif
+#endif /* GNU */
 
 void
 db_task_name(
-	task_t		task)
+	const task_t task)
 {
-	register char *p;
-	register int n;
-	unsigned vaddr, kaddr;
+	char *p;
+	int n;
+	vm_offset_t vaddr, kaddr;
 	unsigned sp;
 
-	if (task->map->pmap == kernel_pmap) {
-		db_printf(DB_GNUMACH_TASK_NAME);
+	if (task->name[0]) {
+		db_printf("%s", task->name);
 		return;
 	}
 
@@ -758,12 +773,12 @@ db_task_name(
 	vaddr = (sp & ~(INTEL_PGBYTES - 1)) + INTEL_PGBYTES;
 	while (1) {
 		if (db_user_to_kernel_address(task, vaddr, &kaddr, 0) < 0)
-			return FALSE;
+			return;
 		if (looks_like_command(task, (char*) kaddr))
 			break;
 		vaddr += INTEL_PGBYTES;
 	}
-#else
+#else /* GNU */
 	vaddr = DB_USER_STACK_ADDR;
 	kaddr = 0;
 
@@ -781,18 +796,18 @@ db_task_name(
 	    db_printf(DB_NULL_TASK_NAME);
 	    return;
 	}
-#endif
+#endif /* GNU */
 
 ok:
 	n = DB_TASK_NAME_LEN-1;
 #ifdef GNU
 	p = (char *)kaddr;
 	for (; n > 0; vaddr++, p++, n--) {
-#else
+#else /* GNU */
 	p = (char *)kaddr + sizeof(unsigned);
 	for (vaddr += sizeof(int); vaddr < DB_USER_STACK_ADDR && n > 0;
 							vaddr++, p++, n--) {
-#endif
+#endif  /* GNU */
 	    if (vaddr % INTEL_PGBYTES == 0) {
 		(void)db_user_to_kernel_address(task, vaddr, &kaddr, 0);
 		p = (char*)kaddr;
