@@ -30,11 +30,6 @@
 #ifndef	_MACH_MACH_I386_TYPES_H_
 #define	_MACH_MACH_I386_TYPES_H_
 
-/*
- * Array of devices.
- */
-typedef	device_t	*device_list_t;
-
 /*
  * i386 segment descriptor.
  */
@@ -46,4 +41,15 @@ struct descriptor {
 typedef struct descriptor descriptor_t;
 typedef	struct descriptor *descriptor_list_t;
 
+/*
+ * i386 I/O port
+ */
+
+#ifdef MACH_KERNEL
+#include <i386/io_perm.h>
+#else /* MACH_KERNEL */
+typedef unsigned short io_port_t;
+typedef mach_port_t io_perm_t;
+#endif /* MACH_KERNEL */
+
 #endif	/* _MACH_MACH_I386_TYPES_H_ */
