@@ -24,6 +24,8 @@
  * the rights to redistribute these changes.
  */
 
+#include <mach/xen.h>
+
 #include <kern/printf.h>
 #include <stdarg.h>
 
@@ -33,12 +35,10 @@
 
 #include <kern/debug.h>
 
-extern void cnputc();
-void Debugger() __attribute__ ((noreturn));
+#include <machine/loose_ends.h>
+#include <machine/model_dep.h>
 
-#if	MACH_KDB
-extern int db_breakpoints_inserted;
-#endif
+#include <device/cons.h>
 
 #if NCPUS>1
 simple_lock_data_t Assert_print_lock; /* uninited, we take our chances */
@@ -51,7 +51,7 @@ do_cnputc(char c, vm_offset_t offset)
 }
 
 void
-Assert(char *exp, char *file, int line)
+Assert(const char *exp, const char *file, int line)
 {
 #if NCPUS > 1
   	simple_lock(&Assert_print_lock);
@@ -63,22 +63,18 @@ Assert(char *exp, char *file, int line)
 		exp, file, line);
 #endif
 
-#if	MACH_KDB
-	if (db_breakpoints_inserted)
-#endif
 	Debugger("assertion failure");
 }
 
-void Debugger(message)
-	char *	message;
+void SoftDebugger(message)
+	const char *message;
 {
-#if	!MACH_KDB
-	panic("Debugger invoked, but there isn't one!");
-#endif
+	printf("Debugger invoked: %s\n", message);
 
-#ifdef	lint
-	message++;
-#endif	/* lint */
+#if	!MACH_KDB
+	printf("But no debugger, continuing.\n");
+	return;
+#endif
 
 #if	defined(vax) || defined(PC532)
 	asm("bpt");
@@ -93,13 +89,23 @@ void Debugger(message)
 	asm("ta 0x81");
 #endif	/* sun4 */
 
-#if	defined(mips ) || defined(luna88k) || defined(i860) || defined(alpha)
+#if	defined(mips ) || defined(i860) || defined(alpha)
 	gimmeabreak();
 #endif
 
-#ifdef	i386
+#if defined(__i386__)
 	asm("int3");
 #endif
+}
+
+void Debugger(message)
+	const char *message;
+{
+#if	!MACH_KDB
+	panic("Debugger invoked, but there isn't one!");
+#endif
+
+	SoftDebugger(message);
 
 	panic("Debugger returned!");
 }
@@ -115,7 +121,7 @@ const char     		*panicstr;
 int			paniccpu;
 
 void
-panic_init()
+panic_init(void)
 {
 	if (!panic_lock_initialized)
 	{
@@ -124,6 +130,10 @@ panic_init()
 	}
 }
 
+#if ! MACH_KBD
+extern boolean_t reboot_on_panic;
+#endif
+
 /*VARARGS1*/
 void
 panic(const char *s, ...)
@@ -151,10 +161,16 @@ panic(const char *s, ...)
 #endif
 	printf(": ");
 	va_start(listp, s);
-	_doprnt(s, &listp, do_cnputc, 0, 0);
+	_doprnt(s, listp, do_cnputc, 0, 0);
 	va_end(listp);
 	printf("\n");
 
+#if	MACH_KDB
+	Debugger("panic");
+#else
+# ifdef	MACH_HYP
+	hyp_crash();
+# else
 	/* Give the user time to see the message */
 	{
 	  int i = 1000;		/* seconds */
@@ -162,10 +178,8 @@ panic(const char *s, ...)
 	    delay (1000000);	/* microseconds */
 	}
 
-#if	MACH_KDB
-	Debugger("panic");
-#else
-	halt_all_cpus (1);
+	halt_all_cpus (reboot_on_panic);
+# endif	/* MACH_HYP */
 #endif
 }
 
@@ -178,10 +192,21 @@ log(int level, const char *fmt, ...)
 {
 	va_list	listp;
 
-#ifdef lint
-	level++;
-#endif
 	va_start(listp, fmt);
-	_doprnt(fmt, &listp, do_cnputc, 0, 0);
+	_doprnt(fmt, listp, do_cnputc, 0, 0);
 	va_end(listp);
 }
+
+/* GCC references this for stack protection.  */
+unsigned char __stack_chk_guard [ sizeof (vm_offset_t) ] =
+{
+	[ sizeof (vm_offset_t) - 3 ] = '\r',
+	[ sizeof (vm_offset_t) - 2 ] = '\n',
+	[ sizeof (vm_offset_t) - 1 ] = 0xff,
+};
+
+void
+__stack_chk_fail (void)
+{
+	panic("stack smashing detected");
+}
