@@ -31,12 +31,9 @@
 
 #include <stddef.h>
 #include <kern/assert.h>
-#include <kern/macro_help.h>
+#include <kern/macros.h>
 #include <sys/types.h>
 
-#define structof(ptr, type, member) \
-    ((type *)((char *)ptr - offsetof(type, member)))
-
 /*
  * Indexes of the left and right nodes in the children array of a node.
  */
@@ -177,8 +174,8 @@ MACRO_END
  * This macro performs a standard lookup to obtain the insertion point of
  * the given node in the tree (it is assumed that the inserted node never
  * compares equal to any other entry in the tree) and links the node. It
- * then It then checks red-black rules violations, and rebalances the tree
- * if necessary.
+ * then checks red-black rules violations, and rebalances the tree if
+ * necessary.
  *
  * Unlike rbtree_lookup(), the cmp_fn parameter must compare two complete
  * entries, so it is suggested to use two different comparison inline
